package report

import (
	"errors"
	"fmt"
	"hongze/hongze_yb/global"
	"hongze/hongze_yb/models/response"
	"hongze/hongze_yb/models/tables/chart_permission_search_key_word_mapping"
	"hongze/hongze_yb/models/tables/rddp/classify"
	"hongze/hongze_yb/models/tables/rddp/report"
	"hongze/hongze_yb/services/company"
	"hongze/hongze_yb/services/user"
	"hongze/hongze_yb/utils"
	"strings"
)

// GetClassListByClassifyId 查询二级分类列表
func GetClassListByClassifyId(user user.UserInfo, classifyIdFirst int) (list []*response.ClassifyListItem, err error) {
	var errMsg string
	defer func() {
		if err != nil {
			global.LOG.Critical(fmt.Sprintf("GetClassListByClassifyName: userId=%d, err:%s， errMsg:%s", user.UserID, err.Error(), errMsg))
		}
	}()

	classifyInfo, err := classify.GetByClassifyId(classifyIdFirst)
	if err != nil {
		errMsg = err.Error()
		err = errors.New("分类查询出错")
		return
	}

	if classifyInfo.Id == 0 {
		err = errors.New("分类不存在")
		return
	}

	if classifyInfo.ParentId != 0 {
		err = errors.New("不允许查询二级分类")
		return
	}

	classifyList, err := classify.GetListByPid(classifyInfo.Id)
	if err != nil {
		errMsg = err.Error()
		err = errors.New("二级分类查询出错")
		return
	}

	//查询二级分类中最新的报告ID
	reportList, err := report.GetLatestReportsByClassifyIdFirst(classifyIdFirst)
	if err != nil && err != utils.ErrNoRow {
		errMsg = err.Error()
		err = errors.New("报告查询出错")
		return
	}

	reportMap := make(map[int]*report.Report)
	for _, v := range reportList {
		reportMap[v.ClassifyIdSecond] = v
	}
	for _, item := range classifyList {
		temp := new(response.ClassifyListItem)
		temp.ClassifyIdSecond = item.Id
		temp.ParentId = item.ParentId
		temp.AuthorDescript = item.AuthorDescript
		temp.ReportAuthor = item.ReportAuthor
		temp.HomeImgUrl = item.HomeImgUrl
		temp.ClassifyNameSecond = item.ClassifyName
		if _, ok := reportMap[item.Id]; ok {
			temp.Stage = reportMap[item.Id].Stage
		}
		if classifyInfo.ClassifyName == "权益研报" {
			temp.ProductName = "权益"
		} else {
			temp.ProductName = "FICC"
		}

		list = append(list, temp)
	}
	return
}

func GetClassifyDetail(user user.UserInfo, classifyIdSecond int) (detail *response.ClassifyDetail, err error)  {
	var errMsg string
	defer func() {
		if err != nil {
			global.LOG.Critical(fmt.Sprintf("GetClassifyDetail: userId=%d, err:%s， errMsg:%s", user.UserID, err.Error(), errMsg))
		}
	}()
	classifyInfo, err := classify.GetByClassifyId(classifyIdSecond)
	if err != nil {
		errMsg = err.Error()
		err = errors.New("分类查询出错")
		return
	}

	if classifyInfo.Id == 0 {
		err = errors.New("分类不存在")
		return
	}
	if classifyInfo.ParentId == 0 {
		err = errors.New("只允许查询二级分类")
		return
	}
	//查询权限, 查询当前分类下的最新的报告，
	permissionIds, err := chart_permission_search_key_word_mapping.GetChartPermissionIdsByKeyWord(classifyInfo.ClassifyName)
	if err != nil {
		errMsg = err.Error()
		err = errors.New("分类权限查询出错")
		return
	}
	authOk, permissionCheckInfo, err := company.GetCheckPermission(user.CompanyID, int(user.UserID), permissionIds)
	if err != nil && err != utils.ErrNoRow {
		errMsg = err.Error()
		err = errors.New("权限查询出错")
		return
	}

	detail = new(response.ClassifyDetail)
	detail.ClassifyIdSecond = classifyInfo.Id
	detail.ParentId = classifyInfo.ParentId
	detail.AuthorDescript = classifyInfo.AuthorDescript
	detail.ReportAuthor = classifyInfo.ReportAuthor
	detail.AvatarImgUrl = classifyInfo.AvatarImgUrl
	detail.ClassifyNameSecond = classifyInfo.ClassifyName
	detail.Abstract = classifyInfo.Abstract
	detail.Descript = classifyInfo.Descript
	if  classifyInfo.IsVip > 0{
		detail.VipTitle = "弘则研究董事长"
	}

	detail.PermissionCheck = &permissionCheckInfo
	detail.AuthOk = authOk

	return
}

// GetClassifyReportList 查询二级分类下的报告列表
func GetClassifyReportList(user user.UserInfo, classifyIdSecond int, pageIndex, pageSize int) (list *response.ClassReportList, err error)  {
	var errMsg string
	defer func() {
		if err != nil {
			global.LOG.Critical(fmt.Sprintf("GetClassifyDetail: userId=%d, err:%s， errMsg:%s", user.UserID, err.Error(), errMsg))
		}
	}()
	classifyInfo, err := classify.GetByClassifyId(classifyIdSecond)
	if err != nil {
		errMsg = err.Error()
		err = errors.New("分类查询出错")
		return
	}

	if classifyInfo.Id == 0 {
		err = errors.New("分类不存在")
		return
	}
	if classifyInfo.ParentId == 0 {
		err = errors.New("只允许查询二级分类")
		return
	}
	//查询权限, 查询当前分类下的最新的报告，
	permissionIds, err := chart_permission_search_key_word_mapping.GetChartPermissionIdsByKeyWord(classifyInfo.ClassifyName)

	if err != nil {
		errMsg = err.Error()
		err = errors.New("分类权限查询出错")
		return
	}
	authOk, permissionCheckInfo, err := company.GetCheckPermission(user.CompanyID, int(user.UserID), permissionIds)
	if err != nil && err != utils.ErrNoRow {
		errMsg = err.Error()
		err = errors.New("权限查询出错")
		return
	}
	//查询 report列表
	offset := (pageIndex - 1) * pageSize
	reportList, err := report.GetListByClassifyIdSecond(classifyIdSecond, offset, pageSize)
	if err != nil {
		errMsg = err.Error()
		err = errors.New("查询报告列表出错")
		return
	}
	total, err := report.GetListCountByClassifyIdSecond(classifyIdSecond)
	if err != nil {
		errMsg = err.Error()
		err = errors.New("查询报告总数出错")
		return
	}

	var newList []*response.ClassReportListItem
	if len(reportList) > 0 {
		for _, reportInfo := range reportList {
			reportItem := new(response.ClassReportListItem)
			reportItem.ReportId = reportInfo.Id
			reportItem.Title = reportInfo.Title
			reportItem.PublishTime = reportInfo.PublishTime
			reportItem.ClassifyNameFirst = reportInfo.ClassifyNameFirst
			reportItem.ClassifyIdFirst = reportInfo.ClassifyIdFirst
			reportItem.ClassifyIdSecond = reportInfo.ClassifyIdSecond
			reportItem.ClassifyNameSecond = reportInfo.ClassifyNameSecond
			reportItem.Stage = reportInfo.Stage
			reportItem.Abstract = reportInfo.Abstract
			reportItem.Author = reportInfo.Author
			reportItem.ReportImgUrl = classifyInfo.ReportImgUrl
			if authOk {
				reportItem.VideoUrl = reportInfo.VideoUrl
			}
			newList = append(newList, reportItem)
		}
	}
	list = new(response.ClassReportList)
	list.Paging = response.GetPaging(pageIndex, pageSize, int(total))
	list.List = newList
	list.PermissionCheck = &permissionCheckInfo
	list.AuthOk = authOk

	return
}

// GetClassifySecondSimpleList 获取二级分类简写名称列表
func GetClassifySecondSimpleList(user user.UserInfo, classifyIdFirst int) (list []*response.ClassifySimpleListItem, err error) {
	var errMsg string
	defer func() {
		if err != nil {
			global.LOG.Critical(fmt.Sprintf("GetClassifySecondSimpleList: userId=%d, err:%s， errMsg:%s", user.UserID, err.Error(), errMsg))
		}
	}()

	classifyParent, err := classify.GetByClassifyId(classifyIdFirst)
	if err != nil {
		errMsg = err.Error()
		err = errors.New("分类查询出错")
		return
	}
	if classifyParent.Id == 0 {
		err = errors.New("该分类不存在")
		return
	}

	simpleList, err := classify.GetListByPid(classifyIdFirst)
	if err != nil {
		errMsg = err.Error()
		err = errors.New("二级分类查询出错")
		return
	}

	if len(simpleList) > 0 {
		for _, item := range simpleList {
			temp := new(response.ClassifySimpleListItem)
			temp.ClassifyIdSecond = item.Id
			temp.ClassifyNameSecond = item.ClassifyName
			if index := strings.Index(item.ClassifyName, "双周报"); index > 0{
				temp.ClassifySecondNameSimple = item.ClassifyName[0 : index]
			}else{
				temp.ClassifySecondNameSimple = item.ClassifyName
			}

			list = append(list, temp)
		}
	}

	return
}


// GetClassifyFirstList 获取一级分类列表
func GetClassifyFirstList(user user.UserInfo) (list []*response.ClassifyFirstListItem, err error) {
	var errMsg string
	defer func() {
		if err != nil {
			global.LOG.Critical(fmt.Sprintf("GetClassifyFirstList: userId=%d, err:%s， errMsg:%s", user.UserID, err.Error(), errMsg))
		}
	}()

	classifyParents, err := classify.GetParentList()
	if err != nil {
		errMsg = err.Error()
		err = errors.New("分类查询出错")
		return
	}
	if len(classifyParents) == 0 {
		err = errors.New("分类不存在")
		return
	}
	classifyMap := make(map[string]bool)
	// TODO 确认正式环境名称是否一致
	classifyMap["需求报告"] = true
	classifyMap["宏观报告"] = true
	classifyMap["日度点评"] = true
	classifyMap["数据点评"] = true
	classifyMap["行业调研"] = true
	classifyMap["碳市场价格周报"] = true
	classifyMap["海外视角"] = true
	classifyMap["百家谈"] = true
	for _, item := range classifyParents {
		temp := new(response.ClassifyFirstListItem)
		temp.ClassifyIdFirst = item.Id
		temp.ClassifyNameFirst = item.ClassifyName
		// TODO ficc菜单图标
		temp.IconImgUrl = ""
		if _, ok := classifyMap[item.ClassifyName]; ok {
			if item.ClassifyName == "需求报告" || item.ClassifyName == "宏观报告" {
				classifySecond, tErr := classify.GetFirstByPid(item.Id)
				if tErr == nil && classifySecond.Id > 0 {
					temp.ClassifyIdSecond = classifySecond.Id
					temp.ClassifyNameSecond = classifySecond.ClassifyName
					temp.RedirectType = 3
				}
			}else{
				temp.RedirectType = 1
			}
		}else{
			temp.RedirectType = 2
		}

		list = append(list, temp)
	}

	return
}


